/**
 * OW2 Util
 * Copyright (C) 2008 SERLI
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: Maven2ResourceFetcher.java 5077 2009-07-29 14:09:04Z fornacif $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.plan.fetcher.impl.maven2;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import org.ow2.util.file.FileUtils;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.plan.bindings.deploymentplan.DeploymentHelper;
import org.ow2.util.plan.bindings.deploymentplan.maven2.Maven2Deployment;
import org.ow2.util.plan.bindings.exceptions.InvalidDeploymentException;
import org.ow2.util.plan.bindings.exceptions.InvalidRepositoryException;
import org.ow2.util.plan.bindings.repository.Repository;
import org.ow2.util.plan.bindings.repository.RepositoryKind;
import org.ow2.util.plan.fetcher.api.IResourceFetcher;
import org.ow2.util.plan.fetcher.api.exceptions.FetcherException;
import org.ow2.util.plan.fetcher.api.exceptions.ResourceFetcherNotResolvedException;
import org.ow2.util.plan.fetcher.api.exceptions.ResourceNotFoundException;
import org.ow2.util.plan.fetcher.impl.AbsResourceFetcherImpl;
import org.ow2.util.url.URLUtils;

/**
 * @author Mickael LEDUQUE
 * @author Francois Fornaciari
 */
public class Maven2ResourceFetcher extends AbsResourceFetcherImpl implements IResourceFetcher {

    /**
     * The logger.
     */
    private static final Log logger = LogFactory.getLog(Maven2ResourceFetcher.class);

    /**
     * The local copy of the resource.
     */
    private File localFile = null;

    /**
     * The URL of the resource.
     */
    private URL foundURL = null;

    /**
     * Last modification time known.
     */
    private long lastModified;

    /**
     * {@inheritDoc}
     */
    public RepositoryKind getSupportedRepositoryType() {
        return RepositoryKind.MAVEN_2;
    }

    /**
     * {@inheritDoc}
     */
    public File getResource() throws ResourceFetcherNotResolvedException {
        if (this.localFile == null) {
            throw new ResourceFetcherNotResolvedException();
        }
        return this.localFile;
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasChanged() throws FetcherException {

        URLConnection connection = null;
        try {
            connection = foundURL.openConnection();
            connection.setReadTimeout(TIMEOUT);
            if (connection.getLastModified() != lastModified) {
                return true;
            }
        } catch (IOException e) {
            logger.debug("Cannot retrieve the last modification date for the URL {0} : {1}", foundURL, e.getMessage());
        } finally {
            try {
                if (connection.getInputStream() != null) {
                    // Close input stream
                    connection.getInputStream().close();
                }
            } catch (IOException e) {
                // Nothing to do
            }
        }

        return false;
    }

    /**
     * {@inheritDoc}
     */
    public void resolve() throws FetcherException {
        if (this.deployment == null) {
            throw new FetcherException(new InvalidDeploymentException("Null deployment"));
        }
        if (!(this.deployment instanceof Maven2Deployment)) {
            throw new FetcherException(new InvalidDeploymentException("Bad deployment type: "
                    + this.deployment.getClass().getName()));
        }
        Maven2Deployment maven2Deployment = (Maven2Deployment) this.deployment;

        logger.debug("Trying to resolve resource {0}", maven2Deployment.toString());

        // Get artifact properties
        String groupId = maven2Deployment.getGroupId();
        String artifactId = maven2Deployment.getArtifactId();
        String version = maven2Deployment.getVersion();
        String type = maven2Deployment.getType();
        String classifier = maven2Deployment.getClassifier();

        if (groupId == null || artifactId == null || version == null) {
            throw new FetcherException(new InvalidDeploymentException("Incomplete maven description: " + maven2Deployment));
        }

        String repositoryRef = null;
        try {
            repositoryRef = DeploymentHelper.getRepositoryRef(maven2Deployment);
        } catch (InvalidDeploymentException e) {
            logger.error(e.getMessage());
        }

        // Default artifact type is jar.
        if (type == null) {
            type = "jar";
        }

        localFile = null;
        if (repositoryRef != null) {
            Repository repository = getRepositoryManager().getRepositoryById(repositoryRef);
            if (repository == null) {
                throw new FetcherException(new InvalidRepositoryException("Undefined repository " + repositoryRef));
            }
            resolveArtifact(repository, groupId, artifactId, version, classifier, type);
        } else {
            // Parse the (maven2) repository list
            for (Repository repository : this.getRepositoryManager()) {
                if (repository.getType() == RepositoryKind.MAVEN_2) {
                    resolveArtifact(repository, groupId, artifactId, version, classifier, type);
                    if (localFile != null && localFile.exists()) {
                        // The resource was found, we can put an end to the search.
                        break;
                    }
                }
            }
        }

        if (localFile == null || !localFile.exists()) {
            throw new FetcherException(new ResourceNotFoundException(maven2Deployment.toString()));
        }
    }

    /**
     * Try to resolve the requested artifact on the given repository.
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @param classifier Artifact's classifier (may be null)
     * @param type Artifact type
     */
    private void resolveArtifact(final Repository repository, final String groupId, final String artifactId,
            final String version, final String classifier, final String type) throws FetcherException {
        URL repositoryURL = null;
        try {
            repositoryURL = new URL(repository.getUrl());
        } catch (MalformedURLException e) {
            logger.debug("Invalid repository URL {0}", repository);
        }

        if (repositoryURL == null) {
            logger.debug("Invalid repository URL:" + repository);
            return;
        }

        // Get the artifact version to be passed to the artifact URL builder
        String maven2Version = Maven2Utils.getMaven2Version(repositoryURL.toExternalForm(), groupId, artifactId, version,
                TIMEOUT);

        try {
            foundURL = Maven2Utils.getMaven2URL(repositoryURL.toExternalForm(), groupId, artifactId, version, maven2Version, classifier,
                    type);
        } catch (MalformedURLException e) {
            logger.debug("Invalid resource URL:" + e.getMessage());
        }

        // If the repository is local, try to return the local file
        if ("file".equals(repositoryURL.getProtocol())) {
            localFile = URLUtils.urlToFile(foundURL);
        } else {

            // Location of the local copy of the resource.
            localFile = Maven2Utils.getMaven2File(this.localRepositoriesBaseDir + File.separator + repository.getId(), groupId,
                    artifactId, version, maven2Version, classifier, type);
        }

        // Keep the last modification time
        lastModified = localFile.lastModified();

        if ("http".equals(repositoryURL.getProtocol())) {
            // Create a local file only if the file doesn't exist or has changed
            if (!localFile.exists() || hasChanged()) {
                InputStream inputStream = null;

                try {
                    URLConnection connection = foundURL.openConnection();
                    connection.setReadTimeout(TIMEOUT);
                    inputStream = connection.getInputStream();

                    if (!localFile.getParentFile().exists()) {
                        localFile.getParentFile().mkdirs();
                    }

                    // Create the local file
                    logger.info("Downloading {0} ({1}Ko)", localFile.getName(), connection.getContentLength() / 1000);
                    FileUtils.dump(inputStream, localFile);

                    // The local file has the same last modified date than the remote resource.
                    // This is useful to check if the remote resource has been modified since the last copy.
                    localFile.setLastModified(connection.getLastModified());

                    // Check the file integrity
                    if (verifyChecksum()) {
                        logger.debug("Resource {0} successfully downloaded", foundURL);
                    } else {
                        // The file is corrupted, delete it
                        FileUtils.delete(localFile);
                        logger.error("Bad checksum: file {0} is corrupted", localFile);
                    }
                } catch (Exception e) {
                    localFile.getParentFile().delete();
                    logger.debug("Exception while copying data from resource {0} : {1}", foundURL, e.getMessage());
                } finally {
                    if (inputStream != null) {
                        try {
                            // Close input stream
                            inputStream.close();
                        } catch (IOException e) {
                            // Nothing to do
                        }
                    }
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    public void update() throws FetcherException {
        this.resolve();
    }

    /**
     * Verify that the checksums of the remote resource and the local copy are equal.
     */
    private boolean verifyChecksum() {
        // The URL of the resource that contains the checksum follows this pattern: [artifactURL].md5
        String remoteChecksumPath = foundURL.toExternalForm().concat(".md5");

        InputStream inputStream = null;
        try {
            // Calculate the MD5 checksum of the local file.
            String localChecksum = FileUtils.calculateChecksum(localFile, "MD5");
            // Get the MD5 checksum of the remote resource.
            URL checksumURL = new URL(remoteChecksumPath);
            URLConnection connection = checksumURL.openConnection();
            connection.setReadTimeout(TIMEOUT);
            inputStream = connection.getInputStream();
            String remoteChecksum = Maven2Utils.readChecksum(inputStream);
            // Are both checksums equal ?
            return (localChecksum.equals(remoteChecksum));
        } catch (Exception e) {
            logger.debug(e.getMessage());
            return false;
        } finally {
            if (inputStream != null) {
                try {
                    // Close input stream
                    inputStream.close();
                } catch (IOException e) {
                    // Nothing to do
                }
            }
        }
    }
}
