/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Maven2Utils.java 5077 2009-07-29 14:09:04Z fornacif $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.plan.fetcher.impl.maven2;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;

/**
 * Maven2 Utilities.
 * @author Francois Fornaciari
 */
public final class Maven2Utils {

    /**
     * Private empty constructor.
     */
    private Maven2Utils() {
    }

    /**
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @param classifier Artifact's classifier (may be null)
     * @param type Artifact type
     * @return The resource location
     */
    public static String getMaven2Location(final String repository, final String groupId, final String artifactId,
            final String version, final String maven2Version, final String classifier, final String type) {

        // Creates a File from artifact info
        String appendedClassifier = "";
        if (classifier != null) {
            // Handle classifier if any
            appendedClassifier = appendedClassifier.concat("-").concat(classifier);
        }

        String path = constructDirectoryPath(groupId, artifactId, version);
        String location = repository.concat("/").concat(
                path.concat("/").concat(artifactId).concat("-").concat(maven2Version).concat(appendedClassifier).concat(".").concat(
                        type));
        return location;
    }

    /**
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @param version Maven2 version
     * @param classifier Artifact's classifier (may be null)
     * @param type Artifact type
     * @return The resource file
     */
    public static File getMaven2File(final String repository, final String groupId, final String artifactId,
            final String version, final String maven2Version, final String classifier, final String type) {
        return new File(getMaven2Location(repository, groupId, artifactId, version, maven2Version, classifier, type));
    }

    /**
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @param classifier Artifact's classifier (may be null)
     * @param type Artifact type
     * @return The resource URL
     * @throws MalformedURLException
     */
    public static URL getMaven2URL(final String repository, final String groupId, final String artifactId,
            final String version, final String maven2Version, final String classifier, final String type) throws MalformedURLException {
        return new URL(getMaven2Location(repository, groupId, artifactId, version, maven2Version, classifier, type));
    }

    /**
     * Read the checksum from a given input stream.
     * @param inputStream The stream to read
     * @return The checksum of a given input stream
     * @throws IOException If the reading of the checksum fails
     */
    public static String readChecksum(final InputStream inputStream) throws IOException {
        String checksum = "";

        InputStreamReader inputStreamReader = null;
        BufferedReader bufferedReader = null;
        try {
            inputStreamReader = new InputStreamReader(inputStream);
            bufferedReader = new BufferedReader(inputStreamReader);

            // Read the checksum
            checksum = bufferedReader.readLine().trim();
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.reset();
                    // Close buffered reader
                    bufferedReader.close();
                } catch (IOException e) {
                    // Nothing to do
                }
            }
            if (inputStreamReader != null) {
                try {
                    inputStreamReader.read();
                    // Close input stream reader
                    inputStreamReader.close();
                } catch (IOException e) {
                    // Nothing to do
                }
            }
        }

        // Return the first entry of the file
        return checksum.split(" ")[0];
    }

    /**
     * Returns the artifact version to be passed to the artifact URL builder May
     * be a timestamped version for SNAPSHOT artifacts.
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @return The artifact version to be passed to the artifact URL builder
     */
    public static String getMaven2Version(final String repository, final String groupId, final String artifactId,
            final String version, final int timeout) {
        String maven2Version = version;

        if (version.endsWith("SNAPSHOT")) {
            String path = constructDirectoryPath(groupId, artifactId, version);
            InputStream inputStream = null;
            try {
                URL mavenMetadataURL = new URL(repository.concat(path.concat("/")).concat("maven-metadata.xml"));
                URLConnection connection = mavenMetadataURL.openConnection();
                connection.setReadTimeout(timeout);
                inputStream = connection.getInputStream();

                DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
                DocumentBuilder parser = documentBuilderFactory.newDocumentBuilder();
                Document document = parser.parse(inputStream);

                // Read timestamped version
                String timestamp = document.getElementsByTagName("timestamp").item(0).getTextContent();
                String buildNumber = document.getElementsByTagName("buildNumber").item(0).getTextContent();

                maven2Version = version.replace("SNAPSHOT", timestamp.concat("-").concat(buildNumber));
            } catch (Exception e) {
                // If the maven-metadata.xml file is not found or
                // if it doesn't contain the timestamp and the buildNumber elements
                // In both cases we have to return the SNAPSHOT version
            } finally {
                if (inputStream != null) {
                    try {
                        // Close input stream
                        inputStream.close();
                    } catch (IOException e) {
                        // Nothing to do
                    }
                }
            }
        }

        return maven2Version;
    }

    /**
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @return transform the artifact coordinates into a m2 repository path
     */
    private static String constructDirectoryPath(final String groupId, final String artifactId, final String version) {
        // Transform groupId '.' into '/'
        String transformed = groupId.replace('.', '/');
        return transformed.concat("/").concat(artifactId).concat("/").concat(version);
    }
}
